<?php

namespace CRM\Controller;

use CRM\Util\Utilities;
use CRM\Traits\TResponse;
use CRM\Service\CrmNovoDashboardService;
use Zeedhi\Framework\DTO\Request;

class NovoDashboardController
{

    protected $DashboardService;

    use TResponse;

    public function __construct(CrmNovoDashboardService $DashboardService)
    {
        $this->DashboardService = $DashboardService;
    }

    public function getTasks()
    {
        try {
            $data = $this->DashboardService->getTasks();
            $this->sendResponse($data, 200);

        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTasksForToday()
    {
        try {
            $data = $this->DashboardService->getTasksCards('hoje');
            $this->sendResponse($data, 200);

        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTasksOutdated()
    {
        try {
            $data = $this->DashboardService->getTasksCards('atrasadas');
            $this->sendResponse($data, 200);

        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTasksFuture()
    {
        try {
            $data = $this->DashboardService->getTasksCards('futuras');
            $this->sendResponse($data, 200);

        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOportunities()
    {
        try {
            $data = $this->DashboardService->getOportunities(); 
            
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getAgenda()
    {
        try {
            $data = [
                'ligacoes' => "15",
                'reunioes' => "01",
            ];
            
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLeads(Request $request)
    {
        try {
            $data = $this->DashboardService->getLeads($request->getParameters());      
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLosts(Request $request)
    {
        try {
            $data = $this->DashboardService->getLosts($request->getParameters());      
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLostsChart(Request $request)
    {
        try {
            $data = $this->DashboardService->getLostsChart($request->getParameters());      
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }


}