<?php

namespace CRM\Factory;

use Doctrine\ORM\EntityManager;
use CRM\Traits\TQuery;

class CrmNovoDashboardFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getLeads($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "CRM_CANDIDATO.NRPARCNEGOCIO",
                "FILACONTATO.CDOPERREAL",
                "TO_CHAR(MIN(FILACONTATO.DTPLANFILACONTA), 'DD/MM/YYYY') AS DTPLANFILACONTA",
                "CRM_CANDIDATO.NMCANDIDATO",
                "CRM_CANDIDATO.CDOPERINCLUSAO",
                "CRM_CANDIDATO.DTULTATU",
                "OPERRES.NMOPERADOR",
                "STAT.IDATIVO",
                "COALESCE(STAT.IDSTATUS, '2') AS IDSTATUSCAN",
                "CASE 
                    WHEN COALESCE(TO_NUMBER(STAT.IDSTATUS), 2) = 3 THEN 3
                    ELSE COALESCE(TO_NUMBER(STAT.IDSTATUS), 2) + 1
                END AS IDSTATUSNEXT",
                "TRUNC(MIN(FILACONTATO.DTPLANFILACONTA)) - TRUNC(SYSDATE) AS VENCIMENTO"
            ])
                ->from('CRM_CANDIDATO', 'CRM_CANDIDATO')
                ->innerJoin(
                    'CRM_CANDIDATO',
                    'FILACONTATO',
                    'FILACONTATO',
                    'FILACONTATO.NRPARCNEGOCIO = CRM_CANDIDATO.NRPARCNEGOCIO
                    AND FILACONTATO.CRM_NRORGTRAB = :CRM_NRORGTRAB 
                    AND CRM_CANDIDATO.NRORG = FILACONTATO.NRORG
                    AND FILACONTATO.DTREALFILACONTA IS NULL'
                )
                ->leftJoin(
                    'CRM_CANDIDATO',
                    'CRM_STATUSPARC',
                    'STAT',
                    'CRM_CANDIDATO.NRPARCNEGOCIO = STAT.NRPARCNEGOCIO
                    AND STAT.CRM_NRORGTRAB = :CRM_NRORGTRAB 
                    AND CRM_CANDIDATO.NRORG = STAT.NRORG
                    AND STAT.IDATIVO = :IDATIVO'
                )
                ->leftJoin(
                    'CRM_CANDIDATO',
                    'OPERADOR',
                    'OPERRES',
                    'FILACONTATO.CDOPERREAL = OPERRES.CDOPERADOR'
                )
                ->leftJoin(
                    'CRM_CANDIDATO',
                    'PARCNEGOCIO',
                    'PARCLEAD',
                    'CRM_CANDIDATO.NRPARCNEGOCIO = PARCLEAD.NRPARCNEGOCIO'
                )
                ->where('FILACONTATO.CDOPERINCLUSAO = :OPERADOR')
                ->andWhere('COALESCE(STAT.IDSTATUS, \'2\') != \'S\'')
                ->addGroupBy('FILACONTATO.CDOPERREAL')
                ->addGroupBy('CRM_CANDIDATO.NRPARCNEGOCIO')
                ->addGroupBy('CRM_CANDIDATO.NMCANDIDATO')
                ->addGroupBy('CRM_CANDIDATO.CDOPERINCLUSAO')
                ->addGroupBy('CRM_CANDIDATO.DTULTATU')
                ->addGroupBy('OPERRES.NMOPERADOR')
                ->addGroupBy('STAT.IDATIVO')
                ->addGroupBy('CASE 
                    WHEN COALESCE(TO_NUMBER(STAT.IDSTATUS), 2) = 3 THEN 3
                    ELSE COALESCE(TO_NUMBER(STAT.IDSTATUS), 2) + 1
                END')
                ->addGroupBy('COALESCE(STAT.IDSTATUS, \'2\')');


            $queryBuilder->setParameter('CRM_NRORGTRAB', $params["NRORG"]);
            $queryBuilder->setParameter('OPERADOR', $params["NRPARCPROPRI"]);
            $queryBuilder->setParameter('IDATIVO', 'S');
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getLosts($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "PAR.NMPRINCIPALPARC",
                "DESCRESULEVOL",
                "NRPARCOPORT",
                "SUM(NVL(OP.VROPORT, 0)) AS VALOROPORTUNI",
                "SUM(NVL(OP.VRPREVOPORT, 0)) AS VALORPREVOPORTUNI",
                "TO_CHAR(OP.DTREALFECHA, 'MONTH') AS MES",
                "TO_CHAR(OP.DTREALFECHA, 'YYYY') AS ANO"
            ])
                ->from('CRM_OPORTUNI', 'OP')

                ->innerJoin(
                    'OP',
                    'EVOLCONT',
                    'EV',
                    'OP.NRSEQCONTATO = EV.NRSEQCONTATO 
                 AND EV.DTHREVOLCONT = (
                    SELECT MAX(DTHREVOLCONT) 
                    FROM EVOLCONT 
                    WHERE NRSEQCONTATO = OP.NRSEQCONTATO
                )'
                )

                ->innerJoin(
                    'EV',
                    'CRM_RESULTEVOL',
                    'RE',
                    'EV.NRSEQRESULEVOL = RE.NRSEQRESULEVOL'
                )

                ->leftJoin(
                    'OP',
                    'PARCNEGOCIO',
                    'PAR',
                    'OP.NRPARCORIGEM = PAR.NRPARCNEGOCIO'
                )
                
                ->innerJoin(
                    'OP',
                    'OPERADOR',
                    'OPE',
                    'OP.NRPARCRESP = OPE.NRPARCNEGOCIO'
                )
                

                ->where('RE.IDGANHAPERDE = \'PERDE\'')
                ->andWhere('EXTRACT(MONTH FROM OP.DTREALFECHA) = \'10\'')
                ->andWhere('EXTRACT(YEAR FROM OP.DTREALFECHA) = \'2024\'')
                ->andWhere('OP.NRORG = :P_NRORG')
                ->andWhere('OPE.CDOPERADOR = :OPERADOR')

                ->groupBy('TO_CHAR(OP.DTREALFECHA, \'MONTH\')')
                ->addGroupBy('TO_CHAR(OP.DTREALFECHA, \'YYYY\')')
                ->addGroupBy('DESCRESULEVOL')
                ->addGroupBy('NRPARCOPORT')
                ->addGroupBy('PAR.NMPRINCIPALPARC');

            $queryBuilder->setParameter('P_NRORG', $params["NRORG"]);
            $queryBuilder->setParameter('OPERADOR', $params["NROPERADOR"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getLostsChart($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "RE.DESCRESULEVOL",
                "COUNT(*) AS QUANTIDADE",
                "TO_CHAR(OP.DTREALFECHA, 'MONTH') AS MES",
                "TO_CHAR(OP.DTREALFECHA, 'YYYY') AS ANO"
            ])
                ->from('CRM_OPORTUNI', 'OP')

                ->innerJoin(
                    'OP',
                    'EVOLCONT',
                    'EV',
                    'OP.NRSEQCONTATO = EV.NRSEQCONTATO 
                 AND EV.DTHREVOLCONT = (
                    SELECT MAX(DTHREVOLCONT) 
                    FROM EVOLCONT 
                    WHERE NRSEQCONTATO = OP.NRSEQCONTATO
                )'
                )
                
                ->innerJoin(
                    'EV',
                    'CRM_RESULTEVOL',
                    'RE',
                    'EV.NRSEQRESULEVOL = RE.NRSEQRESULEVOL'
                )

                ->innerJoin(
                    'OP',
                    'OPERADOR',
                    'OPE',
                    'OP.NRPARCRESP = OPE.NRPARCNEGOCIO'
                )

                ->where('RE.IDGANHAPERDE = \'PERDE\'')
                ->andWhere('EXTRACT(MONTH FROM OP.DTREALFECHA) = \'10\'')
                ->andWhere('EXTRACT(YEAR FROM OP.DTREALFECHA) = \'2024\'')
                ->andWhere('OP.NRORG = :P_NRORG')
                ->andWhere('OPE.CDOPERADOR = :OPERADOR')

                ->groupBy('TO_CHAR(OP.DTREALFECHA, \'MONTH\')')
                ->addGroupBy('TO_CHAR(OP.DTREALFECHA, \'YYYY\')')
                ->addGroupBy('DESCRESULEVOL');

            $queryBuilder->setParameter('P_NRORG', $params["NRORG"]);
            $queryBuilder->setParameter('OPERADOR', $params["NROPERADOR"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getTasks($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "COUNT(CASE WHEN DTPLANFILACONTA < TRUNC(SYSDATE) AND DTREALFILACONTA IS NULL THEN 1 END) AS QTDE_ATRASADAS",
                "COUNT(CASE WHEN DTPLANFILACONTA = TRUNC(SYSDATE) AND DTREALFILACONTA IS NULL THEN 1 END) AS QTDE_HOJE",
                "COUNT(CASE WHEN DTPLANFILACONTA > TRUNC(SYSDATE) AND DTREALFILACONTA IS NULL THEN 1 END) AS QTDE_FUTURAS"
            ])
                ->from('FILACONTATO', 'FILACONTATO')
                ->where('CDOPERREAL = :CDOPERREAL')
                ->andWhere('NRORG = :NRORG');


            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            $queryBuilder->setParameter('CDOPERREAL', $params["NROPERADOR"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getTasksCards($params, $taskType)
    {
        try {
            switch ($taskType) {
                case 'atrasadas':
                    $period = 'FILACONTATO.DTPLANFILACONTA < TRUNC(SYSDATE)';
                    break;
                case 'futuras':
                    $period = 'FILACONTATO.DTPLANFILACONTA > TRUNC(SYSDATE)';
                    break;
                case 'hoje':
                default:
                    $period = 'FILACONTATO.DTPLANFILACONTA = TRUNC(SYSDATE)';
            }
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([ "FILACONTATO.*",  
                "PARCNEGOCIO.NMPRINCIPALPARC"
            ])
            ->from('FILACONTATO', 'FILACONTATO')
            ->leftJoin(
                'FILACONTATO',
                'PARCNEGOCIO',
                'PARCNEGOCIO',
                'FILACONTATO.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
            ->where('FILACONTATO.CDOPERREAL = :CDOPERREAL')
            ->andWhere('FILACONTATO.NRORG = :NRORG')
            ->andWhere($period)
            ->andWhere('FILACONTATO.DTREALFILACONTA IS NULL')
            ->setMaxResults(6);

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            $queryBuilder->setParameter('CDOPERREAL', $params["NROPERADOR"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getOportunities($params)
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();
            $queryBuilder->select([
                "COUNT(CASE WHEN TRUNC(OP.DTINCLUSAO, 'MONTH') = TRUNC(SYSDATE, 'MONTH') THEN 1 END) AS QTDE_CORRENTE",
                "COUNT(CASE WHEN TRUNC(OP.DTINCLUSAO, 'MONTH') = ADD_MONTHS(TRUNC(SYSDATE, 'MONTH'), -1) THEN 1 END) AS QTDE_ANTERIOR",
                "TO_CHAR(SYSDATE, 'FMMonth', 'NLS_DATE_LANGUAGE=PORTUGUESE') AS MES_ATUAL",
                "TO_CHAR(ADD_MONTHS(SYSDATE, -1), 'FMMonth', 'NLS_DATE_LANGUAGE=PORTUGUESE') AS MES_ANTERIOR"
            ])
                ->from('CRM_OPORTUNI', 'OP')
                ->innerJoin(
                    'OP',
                    'OPERADOR',
                    'OPE',
                    'OP.NRPARCRESP = OPE.NRPARCNEGOCIO'
                )
                ->where('OPE.CDOPERADOR = :CDOPERADOR')
                ->andWhere('OP.NRORG = :NRORG');

            $queryBuilder->setParameter('NRORG', $params["NRORG"]);
            $queryBuilder->setParameter('CDOPERADOR', $params["NROPERADOR"]);
            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $queryBuilder->getParameters());
            return $result;
        } catch (\Throwable $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }
}
