import { Utilities } from '@/util/Utilities';
import { DialogService, LoadingService, TextInput } from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CrmClassificacaoDeTarefasController {
  public addNewRow({ component }: IEventParam<TekGrid>) {
    const newId = 0;
    component.addNewRow({ NRSEQCLASSIFICATAREFA: newId, IDATIVO: 'S' }, 'start').then(() => {
      setTimeout(() => {
        const nameInput = Metadata.getInstance<TextInput>(`DSCLASSIFICATAREFA_editable_${newId}`);
        nameInput.setFocus();
      }, 200);
    });
  }

  public saveCrmClassificacaoDeTarefas({ component, event }: IEventParam<TekGrid>) {
    event?.preventDefault();
    console.warn(component);
    if (Utilities.validateGrid(component)) {
      LoadingService.show();
      Http.post('/saveCrmClassificacaoDeTarefas', {
        DRG: component.getEditedRows(),
      }).then(() => {
        component.datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        error.showErrorMessage();
      }).finally(() => {
        component.cancelEditedRows();
        LoadingService.hide();
      });
    }
  }

  public messageConfirmDelete({ event }: any) {
    Utilities.showConfirmDeleteMessage(event, 'CrmClassificacaoDeTarefasController.deleteCrmClassificacaoDeTarefas');
  }

  public deleteCrmClassificacaoDeTarefas() {
    const grid = Metadata.getInstance<TekGrid>('tek-grid-crmClassificacaoDeTarefas_grid');
    LoadingService.show();
    Http.post('/deleteCrmClassificacaoDeTarefas', {
      NRS: grid.selectedRows,
    }).then(() => {
      grid.selectedRows = [];
      grid.reload();
      Utilities.showSuccessMessage();
    }).catch((error) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }
}
