import { Utilities } from '@/util/Utilities';
import {
  Button,
  DialogService,
  Form,
  LoadingService,
  Modal,
  ModalService,
  Select, Tabs, TextInput,
} from
  '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class CrmOrganizacaoPorOperadorController {
  private visibilidadeGridDeOrganizacaoPorOperador: boolean = true;

  private currentRow : any = {};

  private modalAdicionarOrganizacaoPorOperador!: Modal;

  private modalAdicionarOperador!: Modal;

  get textFrameOrganizacaoPorOperadorName(): string {
    return this.currentRow.NMORGTRAB;
  }

  public get visibilidadeDoFrameOrganizacaoPorOperador() {
    return !this.visibilidadeGridDeOrganizacaoPorOperador;
  }

  public voltarTekGridOrganizacaoPorOperador(): void {
    this.visibilidadeGridDeOrganizacaoPorOperador = true;
  }

  public async openModalNewOrganizacaoPorOperador() {
    this.modalAdicionarOperador = await ModalService.createFromJson('crm_organizacao_por_operador/modal/crm_organizacao_por_operador_modal_oper_cadastro', true);
    await this.modalAdicionarOperador.show();
  }

  public fecharModalNewOrganizacaoPorOperador() {
    this.modalAdicionarOperador.hide();
    this.modalAdicionarOperador?.destroy();
  }

  public async openModalNewOrgOrganizacaoPorOperador() {
    this.modalAdicionarOperador = await ModalService.createFromJson('crm_organizacao_por_operador/modal/crm_organizacao_por_operador_modal_org_cadastro', true);
    await this.modalAdicionarOperador.show();
  }

  public fecharModalNewOrgOrganizacaoPorOperador() {
    this.modalAdicionarOperador.hide();
    this.modalAdicionarOperador?.destroy();
  }

  public closeModal() {
    this.modalAdicionarOperador.hide();
    this.modalAdicionarOperador?.destroy();
  }

  get textFrameOrganizacaoName(): string {
    return this.currentRow.NMORGTRAB;
  }

  public clickButtonToExpand({ component }: any): void {
    const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
    form.isVisible = !form.isVisible;
    component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
  }

  private carregarOrgTrabOrganizacaoPorOperadorData(row: any): void {
    Metadata.getInstance<TextInput>('modal-cadastro-NrOrgTrab-CRM_NRORGTRAB').value = row.CRM_NRORGTRAB;
  }

  public openModalNewOrgOrganizacaoPorOperadorCadastro({ row }: any) {
    this.visibilidadeGridDeOrganizacaoPorOperador = false;
    this.currentRow = row;
    this.carregarOrgTrabOrganizacaoPorOperadorData(row);
    this.changeActiveTab();
  }

  private carregarTabOrganizacaoPorOperadorData(row: any): void {
    Metadata.getInstance<TextInput>('formOrgTrabOrganizacaoPorOperadorData-CRM_NRORGTRAB').value = row.CRM_NRORGTRAB;
    Metadata.getInstance<TextInput>('formNomOrgOrganizacaoPorOperadorData-NMORGTRAB').value = row.NMORGTRAB;
    Metadata.getInstance<Select>('formAtivOrganizacaoPorOperadorData-IDATIVO').value = row.IDATIVO;
    Metadata.getInstance<TextInput>('formDescOrganizacaoPorOperadorData-DSORG').value = row.DSORG;
  }

  public get visibilidadeFrameOrganizacaoPorOperador() {
    return !this.visibilidadeGridDeOrganizacaoPorOperador;
  }

  public abrirFrameOrganizacaoPorOperador({ row }: any) {
    this.visibilidadeGridDeOrganizacaoPorOperador = false;
    this.currentRow = row;
    this.carregarTabOrganizacaoPorOperadorData(row);
    this.changeActiveTab();
  }

  public changeActiveTab(): void {
    const tabs = Metadata.getInstance<Tabs>('tabsInformationOrganizacaoPorOperador');
    const activeTab = tabs.tabs[tabs.activeTab];

    switch (activeTab.name) {
      case 'tab_operador':
        setTimeout(() => {
          this.setFilterGridOperador();
        }, 100);
        break;
      default:
        // Do nothing
        break;
    }

    switch (activeTab.name) {
      case 'modal-editarOrgOrganizacao':
        setTimeout(() => {
        }, 100);
        break;
      default:
        // Do nothing
        break;
    }
  }

  public setFilterGridOperador() {
    const gridOrgOperador = Metadata.getInstance<TekGrid>('tekgrid-organizacao-tab-oper');
    gridOrgOperador.datasource.setFilter({
      NRORGANIZACAOOPER: this.currentRow.NRORGANIZACAOOPER,

    });
  }

  public openFilter({ component }: any): void {
    const filterButton = Metadata.getInstance<Button>(
      `${component.name}_filterButton`,
    );
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public saveOrgOganizacao() {
    const formulario = Metadata.getInstance<Form>('form-adicionarOrgOrganizacao');
    const gridOrg = Metadata.getInstance<TekGrid>('tek-grid-crmOrganizacaoPorOperador_grid');
    if (Utilities.validateForm(formulario)) {
      LoadingService.show();
      Http.post('/saveOrgOganizacao', {
        CRM_NRORGTRAB: Metadata.getInstance<TextInput>('modal-cadastro-NrOrgTrab-CRM_NRORGTRAB').value,
        NMORGTRAB: Metadata.getInstance<TextInput>('modal-cadastro-Organizacao-NMORGTRAB').value,
        IDATIVO: Metadata.getInstance<Select>('modal-org-cadastro-idAtivo-IDATIVO').value,
        DSORG: Metadata.getInstance<TextInput>('modal-cadastro-DescOrg-DSORG').value,
      }).then(() => {
        gridOrg.datasource.get();
        this.closeModal();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        // eslint-disable-next-line no-console
        console.log(error);
        error.showErrorMessage();
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
    }
  }

  public saveEditOrgOganizacao() {
    const formulario = Metadata.getInstance<Form>('formOrganizacaoPorOperadorData');
    const gridOrg = Metadata.getInstance<TekGrid>('tek-grid-crmOrganizacaoPorOperador_grid');
    if (Utilities.validateForm(formulario)) {
      LoadingService.show();
      Http.post('/saveEditOrgOganizacao', {
        CRM_NRORGTRAB: Metadata.getInstance<TextInput>('formOrgTrabOrganizacaoPorOperadorData-CRM_NRORGTRAB').value,
        NMORGTRAB: Metadata.getInstance<TextInput>('formNomOrgOrganizacaoPorOperadorData-NMORGTRAB').value,
        IDATIVO: Metadata.getInstance<Select>('formAtivOrganizacaoPorOperadorData-IDATIVO').value,
        DSORG: Metadata.getInstance<TextInput>('formDescOrganizacaoPorOperadorData-DSORG').value,
      }).then(() => {
        gridOrg.datasource.get();
        this.closeModal();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        // eslint-disable-next-line no-console
        console.log(error);
        error.showErrorMessage();
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
    }
  }

  public messageConfirmDelete({ event }: any) {
    Utilities.showConfirmDeleteMessage(event, 'CrmOrganizacaoPorOperadorController.deleteOrgOganizacao');
  }

  public deleteOrgOganizacao() {
    const grid = Metadata.getInstance<TekGrid>('tek-grid-crmOrganizacaoPorOperador_grid');
    LoadingService.show();
    Http.post('/deleteOrgOganizacao', {
      NRS: grid.selectedRows,
    }).then(() => {
      grid.selectedRows = [];
      grid.reload();
      Utilities.showSuccessMessage();
    }).catch((error) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  public saveOperOganizacao() {
    const formulario = Metadata.getInstance<Form>('form-adicionarOperOrganizacao');
    const gridOper = Metadata.getInstance<TekGrid>('tekgrid-organizacao-tab-oper');
    if (Utilities.validateForm(formulario)) {
      LoadingService.show();
      Http.post('/saveOperOganizacao', {
        CDOPERADOR: Metadata.getInstance<Select>('modal-cadastro-operador-CDOPER').value,
        CRM_NRORGTRAB: this.currentRow.CRM_NRORGTRAB,
      }).then(() => {
        gridOper.datasource.get();
        this.closeModal();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        error.showErrorMessage();
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      Utilities.alertMessage('Existem campos obrigatórios não preenchidos!', 'error');
    }
  }
}
